@@ -4,6 +4,8 @@
     ~~~~~~~~~~~~~~~~~~
 
     Automated test suite for Frozen-Flask
+    Run with :
+        $ python -m flask_frozen.tests
 
     :copyright: (c) 2010-2012 by Simon Sapin.
     :license: BSD, see LICENSE for more details.
@@ -21,7 +23,8 @@ from unicodedata import normalize
 from warnings import catch_warnings
 
 from flask_frozen import (Freezer, walk_directory,
-    MissingURLGeneratorWarning, MimetypeMismatchWarning, NotFoundWarning)
+    FrozenFlaskWarning, MissingURLGeneratorWarning, MimetypeMismatchWarning,
+    NotFoundWarning, RedirectWarning)
 from flask_frozen import test_app
 
 try:
@@ -100,6 +103,7 @@ class TestFreezer(unittest.TestCase):
     # URL -> expected bytes content of the generated file
     expected_output = {
         u'/': b'Main index /product_5/?revision=b12ef20',
+        u'/redirect/': b'Main index /product_5/?revision=b12ef20',
         u'/admin/': b'Admin index\n'
             b'<a href="/page/I%20l%C3%B8v%C3%AB%20Unicode/">Unicode test</a>\n'
             b'<a href="/page/octothorp/?query_foo=bar#introduction">'
@@ -127,6 +131,7 @@ class TestFreezer(unittest.TestCase):
     # URL -> path to the generated file, relative to the build destination root
     filenames = {
         u'/': u'index.html',
+        u'/redirect/': u'redirect/index.html',
         u'/admin/': u'admin/index.html',
         u'/robots.txt': u'robots.txt',
         u'/favicon.ico': u'favicon.ico',
@@ -156,7 +161,7 @@ class TestFreezer(unittest.TestCase):
     maxDiff = None
 
     def do_extra_config(self, app, freezer):
-        pass # To be overriden
+        pass  # To be overridden
 
     @contextmanager
     def make_app(self):
@@ -206,7 +211,7 @@ class TestFreezer(unittest.TestCase):
     def test_built_urls(self):
         with self.built_app() as (temp, app, freezer, urls):
             self.assertEqual(set(urls), set(self.expected_output))
-            # Make sure it was not accidently used as a destination
+            # Make sure it was not accidentally used as a destination
             default = os.path.join(os.path.dirname(__file__), 'build')
             self.assertTrue(not os.path.exists(default))
 
@@ -293,7 +298,7 @@ class TestFreezer(unittest.TestCase):
 
     def test_warn_on_internal_404(self):
         with self.make_app_with_404() as (temp, app, freezer):
-            # Enable 404 erros ignoring
+            # Enable 404 errors ignoring
             app.config['FREEZER_IGNORE_404_NOT_FOUND'] = True
             # Test warning with 404 errors when we choose to ignore them
             with catch_warnings(record=True) as logged_warnings:
@@ -301,7 +306,31 @@ class TestFreezer(unittest.TestCase):
                 freezer.freeze()
                 self.assertEqual(len(logged_warnings), 1)
                 self.assertEqual(logged_warnings[0].category,
-                                  NotFoundWarning)
+                                 NotFoundWarning)
+
+    def test_error_on_redirect(self):
+        with self.make_app() as (temp, app, freezer):
+            # Enable errors on redirects.
+            app.config['FREEZER_REDIRECT_POLICY'] = 'error'
+            try:
+                freezer.freeze()
+            except ValueError as e:
+                error_msg = "Unexpected status '302 FOUND' on URL /redirect/"
+                assert error_msg in e.args[0]
+            else:
+                assert False, 'Expected ValueError'
+
+    def test_warn_on_redirect(self):
+        with self.make_app() as (temp, app, freezer):
+            # Enable ignoring redirects.
+            app.config['FREEZER_REDIRECT_POLICY'] = 'ignore'
+            # Test warning with 302 errors when we choose to ignore them
+            with catch_warnings(record=True) as logged_warnings:
+                warnings.simplefilter("always")
+                freezer.freeze()
+                self.assertEqual(len(logged_warnings), 1)
+                self.assertEqual(logged_warnings[0].category,
+                                 RedirectWarning)
 
     def test_warn_on_missing_generator(self):
         with self.make_app() as (temp, app, freezer):
@@ -315,7 +344,7 @@ class TestFreezer(unittest.TestCase):
                 freezer.freeze()
                 self.assertEqual(len(logged_warnings), 1)
                 self.assertEqual(logged_warnings[0].category,
-                                  MissingURLGeneratorWarning)
+                                 MissingURLGeneratorWarning)
 
     def test_wrong_default_mimetype(self):
         with self.make_app() as (temp, app, freezer):
@@ -328,7 +357,7 @@ class TestFreezer(unittest.TestCase):
                 freezer.freeze()
                 self.assertEqual(len(logged_warnings), 1)
                 self.assertEqual(logged_warnings[0].category,
-                                  MimetypeMismatchWarning)
+                                 MimetypeMismatchWarning)
 
     def test_default_mimetype(self):
         with self.make_app() as (temp, app, freezer):
@@ -339,7 +368,7 @@ class TestFreezer(unittest.TestCase):
 
     def test_unknown_extension(self):
         with self.make_app() as (temp, app, freezer):
-            @app.route(u'/unkown-extension.fuu')
+            @app.route(u'/unknown-extension.fuu')
             def no_extension():
                 return '42', 200, {'Content-Type': 'application/octet-stream'}
             freezer.freeze()
@@ -363,7 +392,38 @@ class TestFreezer(unittest.TestCase):
                 freezer.freeze()
                 self.assertEqual(len(logged_warnings), 1)
                 self.assertEqual(logged_warnings[0].category,
-                                  MimetypeMismatchWarning)
+                                 MimetypeMismatchWarning)
+
+    def test_skip_existing_files(self):
+        with self.make_app() as (temp, app, freezer):
+            app.config['FREEZER_SKIP_EXISTING'] = True
+            with open(os.path.join(temp, 'skipped.html'), 'w') as f:
+                f.write("6*9")
+            @app.route(u'/skipped.html')
+            def skipped():
+                return '42'
+            freezer.freeze()
+            with open(os.path.join(temp, 'skipped.html')) as f:
+                self.assertEqual(f.read(), '6*9')
+
+
+class TestWarnings(unittest.TestCase):
+    def test_warnings_share_common_superclass(self):
+        with catch_warnings(record=True) as logged_warnings:
+            # ignore all warnings:
+            warnings.simplefilter('ignore')
+            # but don't ignore FrozenFlaskWarning
+            warnings.filterwarnings('always', category=FrozenFlaskWarning)
+            # warn each of our warnings:
+            warnings_frozen_flask = (MissingURLGeneratorWarning,
+                                     MimetypeMismatchWarning,
+                                     NotFoundWarning,
+                                     RedirectWarning)
+            for warning in warnings_frozen_flask:
+                warnings.warn('test', warning)
+            # warn something different:
+            warnings.warn('test', PendingDeprecationWarning)
+            self.assertEqual(len(logged_warnings), len(warnings_frozen_flask))
 
 
 class TestInitApp(TestFreezer):
@@ -387,7 +447,7 @@ class TestBaseURL(TestFreezer):
 
 class TestNonexsistentDestination(TestFreezer):
     def do_extra_config(self, app, freezer):
-        # frozen/htdocs does not exsist in the newly created temp directory,
+        # frozen/htdocs does not exist in the newly created temp directory,
         # the Freezer has to create it.
         app.config['FREEZER_DESTINATION'] = os.path.join(
             app.config['FREEZER_DESTINATION'], 'frozen', 'htdocs')
@@ -401,7 +461,6 @@ class TestServerName(TestFreezer):
         b'/where_am_i/ http://example.net/where_am_i/')
 
 
-
 class TestWithoutUrlForLog(TestFreezer):
     freezer_kwargs = dict(log_url_for=False)
 
